# -*- coding: utf-8 -*-

# (C) Djordjevic Nebojsa <nesh@studioquattro.co.yu> 2005
# You're welcome to redistribute this software under the
# terms of the GNU Library General Public Licence version 2.0
# or, at your option, any higher version.
#
# You can read the complete GNU LGPL in the file COPYING
# which should come along with this software, or visit
# the Free Software Foundation's WEB site http://www.fsf.org
#

""" global i18n functions for use in core and simmilar
    modules
    
    based on mailman's i18n module
"""

##################################################
## DEPENDENCIES ##

import sys
import os
from gettext import GNUTranslations, NullTranslations
import time
import filecache

##################################################
## GLOBALS AND CONSTANTS ##

# current translation
_translation = None

##################################################
## CLASSES ##

class _TranslationCache(filecache.FileCache):
    def do_load(self, path, mode):
        """ return XXXTranslations instead file content """
        
        try:
            # .mo files are binary files, windows requires 'b' flag
            return GNUTranslations(file(path, 'rb'))
        except IOError, err:
            return NullTranslations()
        #
    # do_load
# _TranslationCache

# XXX move to utils.datastructures?
class SafeDict(dict):
    """Dictionary which returns a default value for unknown keys.

    This is used in maketext so that editing templates is a bit more robust.

    from Mailman
    """
    def __getitem__(self, key):
        try:
            return super(SafeDict, self).__getitem__(key)
        except KeyError:
            if isinstance(key, basestring):
                return '%(' + key + ')s'
            else:
                return '<Missing key: %s>' % key
    #
# SafeDict

##################################################
## FUNCTIONS ##

# init cache
_translation_cache = _TranslationCache()

def available_locales(path):
    """ traverse path finding all .mo files in format
        returns: {domain: locale list}
    """

    if path is None:
        return {}

    ret = {}

    for root, dirs, files in os.walk(path):
        for name in files:
            if name.endswith('.mo'): # name is domain name
                domain = name[:-3]

                if domain not in ret:
                    ret[domain] = []

                # path is like <root>/<locale>/<domain>.mo
                locale = os.path.basename(root)
                ret[domain].append(locale)
        # for
    # for

    return ret
# available_locales

def setup_from_request(req):
    """ setup locale using data from django request object """

    # try cookie first
    if 'lang' in req.COOKIES:
        set_language(language=req.COOKIES['lang'])
        return
    #

    # try to find out preffered language from the browser
    accept = req.META.get('HTTP_ACCEPT_LANGUAGE', None)
    if accept is not None:
        set_language(language=_parse_accept(accept))
        return
    #

    # fallback
    # try to get default locale from config
    try:
        import django.conf.settings
        set_language(language=getattr(django.conf.settings, 'DEFAULT_LOCALE', None))
    except ImportError:
        pass
# setup_from_request

def _parse_accept(accept):
    """ parse accept-language header """

    lst = accept.split(',')
    langs = {}

    for l in lst:
        if ';' in l: # have preference
            lang, order = l.split(';')
            langs[int(float(order.strip()[2:]) * 100)] = lang.strip()[:2]
        else:
            langs[100] = l.strip()[:2]
    # for

    return langs[max(langs.keys())]
# parse_accept
    
def set_language(language=None, domain='django', path=None):
    """ load localization file and setup current translation

        locale files are stored in:
            <path>/<language>/<domain>.mo

        parameters:
            language string: locale code ('en', 'en_US', 'sr', ...)
            domain string:   translation domain
            path string:     path where file is located
    """

    global _translation

    if language is None:
        _translation = NullTranslations()
        return
    #
    
    
    if path is None:
        # try to find out location of message files
        try:
            import django.conf.settings
            my_dir = os.path.dirname(__file__)
            # I'm in django.utils, so locale dir is ../../locale/
            path = getattr(django.conf.settings, 'LOCALE_DIR',
                           os.path.join(os.path.abspath(os.path.join(my_dir, '..', '..')), 'locale'))
        except ImportError:
            path = ''
    #
    try:
        _translation = _translation_cache.load(os.path.join(path, language, domain) + '.mo')
    except (OSError, IOError), err:
        from django.conf.settings import DEBUG
        if DEBUG:
            import warnings
            warnings.warn('Translation %r not found for domain %r in %r (%s)' % (language, domain, path, err),
                          RuntimeWarning, 3)
        _translation = NullTranslations() # fallback
    #
# set_language

def get_translation():
    """ return current translation """
    return _translation
#

def set_translation(translation):
    global _translation

    _translation = translation
#


# setup global translation
if _translation is None:
    set_language()
#

def ugettext(s):
    """ Do translation of the given string into current language

        returns: unicode string with translated message
    """
    if (s == '') or (s is None):
        return u''

    frame = sys._getframe(1)
    dict = SafeDict()
    dict.update(frame.f_locals)

    tns = _translation.ugettext(s)
    return tns % dict
# ugettext

def ungettext(singular, plural, n):
    """ use ungettext to get correct plural form """
    frame = sys._getframe(1)
    dict = SafeDict()
    dict.update(frame.f_locals)

    tns = _translation.ungettext(singular, plural, n)
    
    return tns % dict
# ungettext
