# -*- coding: utf-8 -*-

# (C) Djordjevic Nebojsa <nesh@studioquattro.co.yu> 2005
# You're welcome to redistribute this software under the
# terms of the GNU Library General Public Licence version 2.0
# or, at your option, any higher version.
#
# You can read the complete GNU LGPL in the file COPYING
# which should come along with this software, or visit
# the Free Software Foundation's WEB site http://www.fsf.org
#

""" l10n functions

    based on locale python module
"""

##################################################
## DEPENDENCIES ##

import os
from numeric import _Numeric
from decimal import Decimal, ROUND_UP, ROUND_DOWN

##################################################
## GLOBALS AND CONSTANTS ##

##################################################
## CLASSES ##

class _Monetary(_Numeric):
    _FILE_NAME = 'LC_MONETARY'
     
    def __init__(self, path=None):
        super(_Monetary, self).__init__(path)
        
        # set defaults
        args = {}
        if path is not None:
            try:
                args.update(self._load(path, self._FILE_NAME))
            except IOError:
                pass
            #
        #
        self._parse(args)
    # __init__

    def _parse(self, data):
        # defaults are from US locale
        # rename options for numeric parser
        if 'mon_decimal_point' in data:
            data['decimal_point'] = data['mon_decimal_point']

        if 'mon_thousands_sep' in data:
            data['thousands_sep'] = data['mon_thousands_sep']
        data['thousands_sep'] = data.get('thousands_sep', ',')

        if 'mon_grouping' in data:
            data['grouping'] = data['mon_grouping']
        data['grouping'] = data.get('grouping', '3')
        
        # parse numeric data
        super(_Monetary, self)._parse(data)

        self.int_curr_symbol = data.get('int_curr_symbol', 'USD')
        if len(self.int_curr_symbol) < 4:
            self.int_curr_symbol_sep = ' '
        else:
            self.int_curr_symbol_sep = self.int_curr_symbol[-1]
            self.int_curr_symbol = self.int_curr_symbol[:-1]
        #

        self.currency_symbol = data.get('currency_symbol', '$')
        self.positive_sign = data.get('positive_sign', '')
        self.negative_sign = data.get('negative_sign', '-')
        self.int_frac_digits = int(data.get('int_frac_digits', '2'))
        self.frac_digits = int(data.get('frac_digits', '2'))
        self.p_cs_precedes = data.get('p_cs_precedes', '1').strip() != '0'
        self.p_sep_by_space = int(data.get('p_sep_by_space', '2'))
        self.n_cs_precedes = data.get('n_cs_precedes', '1').strip() != '0'
        self.n_sep_by_space = int(data.get('n_sep_by_space', '2'))
        self.p_sign_posn = int(data.get('p_sign_posn', '3'))
        self.n_sign_posn = int(data.get('n_sign_posn', '3'))
        self.debit_sign = data.get('debit_sign', 'DB')
        self.credit_sign = data.get('credit_sign', 'CR')
        self.left_parenthesis = data.get('left_parenthesis', '(')
        self.right_parenthesis = data.get('left_parenthesis', ')')
        
    # _parse
    
    def _str(self, val, international=False):
        """ from example from python decimal manual """
        
        if not isinstance(val, Decimal):
            raise ValueError('using floats is dangerous with money :), use Decimal')
        #
        
        if international:
            places = -self.int_frac_digits
            curr = self.int_curr_symbol
        else:
            places = -self.int_frac_digits
            curr = self.currency_symbol
        #
        dp = self.decimal_point
        positive = val > 0

        q = Decimal((0, (1,), places))
        number = val.quantize(q, rounding=ROUND_UP)

        # group numbers
        _, digits, exp = number.as_tuple()
        digits = ''.join([str(d) for d in digits])
        integer, _ = self._group(digits[:exp])
        result = integer + dp + digits[exp:]
        
        if positive:
            precedes = self.p_cs_precedes
            sign_posn = self.p_sign_posn
            sep_by_space = self.p_sep_by_space
        else:
            precedes = self.n_cs_precedes
            sign_posn = self.n_sign_posn
            sep_by_space = self.n_sep_by_space
        #

        result = self._formatcurr(precedes, sign_posn, sep_by_space, curr, positive, result)
        return result
    #

    def _formatcurr(self, precedes, sign_posn, sep_by_space, curr, positive, result):
        sign = (positive and [self.positive_sign] or [self.negative_sign])[0]
        fmt = '%(result)s'
        if sign_posn == 0:
            if precedes:
                if sep_by_space == 0:
                    fmt = '%(curr)s%(result)s'
                elif sep_by_space == 1:
                    fmt = '%(curr)s %(result)s'
                elif sep_by_space == 2:
                    fmt = '%(curr)s%(result)s'
            else:
                if sep_by_space == 0:
                    fmt = '%(result)s%(curr)s'
                elif sep_by_space == 1:
                    fmt = '%(result)s %(curr)s'
                elif sep_by_space == 2:
                    fmt = '%(result)s %(curr)s'
            # if precedes
            fmt = self.left_parenthesis + fmt + self.right_parenthesis
        #
        elif sign_posn == 1:
            if precedes:
                if sep_by_space == 0:
                    fmt = '%(sign)s%(curr)s%(result)s'
                elif sep_by_space == 1:
                    fmt = '%(sign)s%(curr)s %(result)s'
                elif sep_by_space == 2:
                    fmt = '%(sign)s %(curr)s%(result)s'
            else:
                if sep_by_space == 0:
                    fmt = '%(sign)s%(result)s%(curr)s'
                elif sep_by_space == 1:
                    fmt = '%(sign)s%(result)s %(curr)s'
                elif sep_by_space == 2:
                    fmt = '%(sign)s%(result)s %(curr)s'
            # if precedes
        elif sign_posn == 2:
            if precedes:
                if sep_by_space == 0:
                    fmt = '%(curr)s%(result)s%(sign)s'
                elif sep_by_space == 1:
                    fmt = '%(curr)s %(result)s%(sign)s'
                elif sep_by_space == 2:
                    fmt = '%(curr)s%(result)s %(sign)s'
            else:
                if sep_by_space == 0:
                    fmt = '%(result)s%(curr)s%(sign)s'
                elif sep_by_space == 1:
                    fmt = '%(result)s %(curr)s%(sign)s'
                elif sep_by_space == 2:
                    fmt = '%(result)s %(curr)s %(sign)s'
            # if precedes
        elif sign_posn == 3:
            if precedes:
                if sep_by_space == 0:
                    fmt = '%(sign)s%(curr)s%(result)s'
                elif sep_by_space == 1:
                    fmt = '%(sign)s%(curr)s %(result)s'
                elif sep_by_space == 2:
                    fmt = '%(sign)s %(curr)s%(result)s'
            else:
                if sep_by_space == 0:
                    fmt = '%(result)s%(sign)s%(curr)s'
                elif sep_by_space == 1:
                    fmt = '%(result)s %(sign)s%(curr)s'
                elif sep_by_space == 2:
                    fmt = '%(result)s%(sign)s %(curr)s'
            # if precedes
        elif sign_posn == 4:
            if precedes:
                if sep_by_space == 0:
                    fmt = '%(curr)s%(sign)s%(result)s'
                elif sep_by_space == 1:
                    fmt = '%(curr)s%(sign)s %(result)s'
                elif sep_by_space == 2:
                    fmt = '%(curr)s %(sign)s%(result)s'
            else:
                if sep_by_space == 0:
                    fmt = '%(result)s%(curr)s%(sign)s'
                elif sep_by_space == 1:
                    fmt = '%(result)s %(curr)s%(sign)s'
                elif sep_by_space == 2:
                    fmt = '%(result)s%(curr)s %(sign)s'
            # if precedes
        #
        return (fmt % locals()).strip() # if sign is '' get rid of whitespaces
    #

    def decimal(self, string):
        # get rid of currency symbols
        if self.int_curr_symbol:
            string = string.replace(self.int_curr_symbol, '')
        if self.currency_symbol:
            string = string.replace(self.currency_symbol, '')

        # get rid of parenthesis
        if self.left_parenthesis:
            string = string.replace(self.left_parenthesis, '')
        if self.right_parenthesis:
            string = string.replace(self.right_parenthesis, '')

        # get rid of grouping
        if self.thousands_sep:
            string = string.replace(self.thousands_sep, '')
        # replace decimal point
        if self.decimal_point:
            string = string.replace(self.decimal_point, '.')

        # positive sign
        if self.positive_sign:
            string = string.replace(self.positive_sign, '')
        #

        # negative sign
        if self.negative_sign and (self.negative_sign in string):
            string = '-' + string.replace(self.negative_sign, '')
        #

        string = string.replace(' ', '') # get rid of all spaces
        return Decimal(string)
    #
    
    def str(self, val):
        return self._str(val)
    # str

    def int_str(self, val):
        return self._str(val, True)
    #
# _Monetary
