# -*- coding: utf-8 -*-

# (C) Djordjevic Nebojsa <nesh@studioquattro.co.yu> 2005
# You're welcome to redistribute this software under the
# terms of the GNU Library General Public Licence version 2.0
# or, at your option, any higher version.
#
# You can read the complete GNU LGPL in the file COPYING
# which should come along with this software, or visit
# the Free Software Foundation's WEB site http://www.fsf.org
#

""" l10n functions

    based on locale python module
"""

##################################################
## DEPENDENCIES ##

import os
from baselocale import _BaseLC
import decimal

##################################################
## GLOBALS AND CONSTANTS ##

##################################################
## CLASSES ##

class _Numeric(_BaseLC):
    _FILE_NAME = 'LC_NUMERIC'
    
    def __init__(self, path=None):
        super(_Numeric, self).__init__(path)

        # set defaults
        args = {}
        if path is not None:
            try:
                args.update(self._load(path, self._FILE_NAME))
            except IOError:
                pass
        #
        self._parse(args)
    # __init__
    
    def _parse(self, data):
        self.decimal_point = data.get('decimal_point', '.')
        self.thousands_sep = data.get('thousands_sep', '')
        
        if 'grouping' in data:
            if ';' in data['grouping']:
                grp = data['grouping'].split(';')
                self.grouping = []
                for g in grp:
                    g = int(g)
                    if g == -1:
                        self.grouping.append(None) # end marker
                        break # end
                    #
                    self.grouping.insert(0, g)
                #
            else:
                self.grouping = [int(data['grouping'])]
            #
        else:
            self.grouping = [None]
        #
    # _parse
    
    # LOCALE FUNCTIONS
    def _group(self, s):
        """ perform the grouping from right to left

            from locale.py
            Author: Martin von Loewis
        """
        grouping = self.grouping[:] # copy
        
        if not grouping: return (s, 0)

        result = ""
        seps = 0
        spaces = ""

        if s[-1] == ' ':
            sp = s.find(' ')
            spaces = s[sp:]
            s = s[:sp]
        #
        
        while s and grouping:
            # if grouping is -1, we are done
            if grouping[0] is None:
                break
            
            # 0: re-use last group ad infinitum
            elif grouping[0] != 0:
                #process last group
                group = grouping[0]
                if len(grouping) > 1:
                    grouping = grouping[1:]
                
            if result:
                result=s[-group:] + self.thousands_sep + result
                seps += 1
            else:
                result=s[-group:]
                
            s = s[:-group]
            if s and s[-1] not in "0123456789":
                # the leading string is only spaces and signs
                return s + result + spaces, seps
        # while
        
        if not result:
            return s + spaces, seps
        
        if s:
            result = s + self.thousands_sep + result
            seps += 1
        #
        return result + spaces, seps
    # _group


    def format(self, f, val, grouping=False):
        """ Formats a value in the same way that the % formatting would use,
            but takes the current locale into account.
            Grouping is applied if the third parameter is true.

            from locale.py
            Author: Martin von Loewis
        """
        #if isinstance(val, decimal.Decimal):
        #    # f represents number of decimals not a format string
        #    result
        result = f % val
        fields = result.split(".")
        seps = 0
        
        if grouping:
            fields[0], seps = self._group(fields[0])
            
        if len(fields) == 2:
            result = fields[0] + self.decimal_point + fields[1]
        elif len(fields) == 1:
            result = fields[0]
        else:
            raise ValueError, "Too many decimal points in result string"

        while seps:
            # If the number was formatted for a specific width, then it
            # might have been filled with spaces to the left or right. If
            # so, kill as much spaces as there where separators.
            # Leading zeroes as fillers are not yet dealt with, as it is
            # not clear how they should interact with grouping.
            sp = result.find(" ")
            if sp == -1: break
            result = result[:sp] + result[sp + 1:]
            seps -= 1
        return result
    # format

    def str(self, val):
        """ Convert float to integer, taking the locale into account.

            from locale.py
            Author: Martin von Loewis
        """
        return self.format("%.12g", val)
    # srt

    def _atox(self, string, func):
        """ Parses a string according to the locale settings.

            from locale.py
            Author: Martin von Loewis
        """
        #First, get rid of the grouping
        ts = self.thousands_sep
        if ts:
            string = string.replace(ts, '')
            
        #next, replace the decimal point with a dot
        dd = self.decimal_point
        if dd:
            string = string.replace(dd, '.')
            
        #finally, parse the string
        return func(string)
    # atof

    def atof(self, string):
        """ Converts a string to an float according to the locale settings.

            from locale.py
            Author: Martin von Loewis
        """
        return self._atox(string, float)
    # atof

    def atoi(self, str):
        """ Converts a string to an integer according to the locale settings.

            from locale.py
            Author: Martin von Loewis
        """
        return self._atox(str, int)
    # atoi
# _Numeric
