import unittest

from l10n import _Locale
class TestLocale(unittest.TestCase):
    def setUp(self):
        self.locale = _Locale()
    #

    def test_empty(self):
        loc = self.locale
        self.assertEqual(loc.language, None)
        self.assertEqual(loc.encoding, None)
    #
# TestLocale

from decimal import Decimal
from monetary import _Monetary
# format (p_cs_preceedes, p_sign_posn, p_sep_by_space, 'ret')
_M_DATA = (
    (True, 0, 2, '($1.25)'),
    (True, 0, 1, '($ 1.25)'),
    (True, 0, 0, '($1.25)'),

    (False, 0, 2, '(1.25 $)'),
    (False, 0, 1, '(1.25 $)'),
    (False, 0, 0, '(1.25$)'),

    (True, 1, 2, '+ $1.25'),
    (True, 1, 1, '+$ 1.25'),
    (True, 1, 0, '+$1.25'),

    (True, 2, 2, '$1.25 +'),
    (True, 2, 1, '$ 1.25+'),
    (True, 2, 0, '$1.25+'),

    (True, 3, 2, '+ $1.25'),
    (True, 3, 1, '+$ 1.25'),
    (True, 3, 0, '+$1.25'),

    (True, 4, 2, '$ +1.25'),
    (True, 4, 1, '$+ 1.25'),
    (True, 4, 0, '$+1.25'),

    (False, 1, 2, '+1.25 $'),
    (False, 1, 1, '+1.25 $'),
    (False, 1, 0, '+1.25$'),

    (False, 2, 2, '1.25 $ +'),
    (False, 2, 1, '1.25 $+'),
    (False, 2, 0, '1.25$+'),

    (False, 3, 2, '1.25+ $'),
    (False, 3, 1, '1.25 +$'),
    (False, 3, 0, '1.25+$'),

    (False, 4, 2, '1.25$ +'),
    (False, 4, 1, '1.25 $+'),
    (False, 4, 0, '1.25$+'),
    )

class TestMonetary(unittest.TestCase):
    def test_default(self):
        num = _Monetary()
        val = Decimal(123456789)
        ret = '$123,456,789.00'
        self.assertEqual(num.str(val), ret)

        val = Decimal(-123456789)
        ret = '- $123,456,789.00'
        self.assertEqual(num.str(val), ret)
    #

    def test_int_curr(self):
        num = _Monetary()
        val = Decimal(123456789)
        ret = 'USD123,456,789.00'
        self.assertEqual(num.int_str(val), ret)

        num.int_curr_symbol = 'TEST'
        ret = 'TEST123,456,789.00'
        self.assertEqual(num.int_str(val), ret)
    #

    def test_currency(self):
        num = _Monetary()
        val = Decimal(123456789)
        ret = '$123,456,789.00'
        self.assertEqual(num.str(val), ret)

        num.currency_symbol = 'T'
        ret = 'T123,456,789.00'
        self.assertEqual(num.str(val), ret)
    #

    def test_decimal(self):
        num = _Monetary()
        ret = Decimal('1.25')
        num.positive_sign = '+'
        
        for _, _, _, val in _M_DATA:
            self.assertEqual(num.decimal(val), ret)
        #

        ret = Decimal('-1.25')
        
        for _, _, _, val in _M_DATA[6:]: # skip first ()
            val = val.replace('+', '-')
            self.assertEqual(num.decimal(val), ret)
        #
    #

    # tests from LC_MONETARY doc's
    def test_pos(self):
        num = _Monetary()
        val = Decimal('1.245234')
        num.positive_sign = '+'

        for preceedes, sign, space, ret in _M_DATA:
            num.p_cs_precedes = preceedes
            num.p_sign_posn = sign
            num.p_sep_by_space = space
            
            self.assertEqual(num.str(val), ret)

            ret = ret.replace('$', 'USD') # int currency test
            self.assertEqual(num.int_str(val), ret)
        # for
    #

    def test_neg(self):
        num = _Monetary()
        val = Decimal('-1.245234')

        for preceedes, sign, space, ret in _M_DATA:
            num.n_cs_precedes = preceedes
            num.n_sign_posn = sign
            num.n_sep_by_space = space
            ret = ret.replace('+', '-')
            
            self.assertEqual(num.str(val), ret)

            ret = ret.replace('$', 'USD') # int currency test
            self.assertEqual(num.int_str(val), ret)
        # for
    #
    # TODO: DB/CR test
# TestMonetary

from numeric import _Numeric
class TestNumeric(unittest.TestCase):
    def test_format(self):
        num = _Numeric()
        val = 123456789
        ret = '123456789'
        self.assertEqual(num.format('%d', val, True), ret)
    #

    def test_str(self):
        num = _Numeric()
        val = 3.14
        ret = '3.14'
        self.assertEqual(num.str(val), ret)
    #

    def test_atoi(self):
        num = _Numeric()
        val = '123456789'
        ret = 123456789
        self.assertEqual(num.atoi(val), ret)

        num.thousands_sep = "."
        val = '123.456.789'
        self.assertEqual(num.atoi(val), ret)
    #

    def test_atof(self):
        num = _Numeric()
        val = '3.14'
        ret = 3.14
        self.assertEqual(num.atof(val), ret)
    #

    def test_decimal_step(self):
        num = _Numeric()
        val = 3.14

        ret = '3.14'
        self.assertEqual(num.str(val), ret)

        ret = '3,14'
        num.decimal_point = ','
        self.assertEqual(num.str(val), ret)
    #

    def test_grouping(self):
        num = _Numeric()
        num.thousands_sep = "'"
        val = 123456789

        num.grouping = [3, None]
        ret = "123456'789"
        self.assertEqual(num.format('%d', val, True), ret)

        num.grouping = [3]
        ret = "123'456'789"
        self.assertEqual(num.format('%d', val, True), ret)

        num.grouping = [3, 2, None]
        ret = "1234'56'789"
        self.assertEqual(num.format('%d', val, True), ret)

        num.grouping = [3, 2]
        ret = "12'34'56'789"
        self.assertEqual(num.format('%d', val, True), ret)

        num.grouping = [None]
        ret = "123456789"
        self.assertEqual(num.format('%d', val, True), ret)

    #
# TestNumeric

if __name__ == '__main__':
    unittest.main()
